part of 'type_result.dart';

@immutable
abstract class TypeResultSomeOf extends TypeResult {
  TypeResultSomeOf(
    super.className, {
    required this.subTypes,
    super.nullable,
  }) : super(isTypeDef: false);

  final Set<TypeResult> subTypes;

  @override
  String? get _builderFactory => null;

  @override
  String? get _serializer {
    if (optimizedSubTypes.length <= 1) {
      return null;
    }

    return '..add(${className}Extension.serializer)';
  }

  late final BuiltSet<TypeResult> optimizedSubTypes = _optimizedSubTypes.toBuiltSet();

  @override
  TypeResult get dartType {
    if (optimizedSubTypes.isEmpty) {
      throw StateError('SomeOf must have at least one sub type.');
    }

    if (optimizedSubTypes.length == 1) {
      return optimizedSubTypes.single;
    }

    final record = optimizedSubTypes.map((final type) {
      final dartType = type.nullableName;
      final dartName = toDartName(dartType);
      final fieldName = toFieldName(dartName, type.className);

      return '$dartType $fieldName';
    });

    return TypeResultBase('({${record.join(',')}})');
  }

  Iterable<TypeResult> get _optimizedSubTypes sync* {
    var optimizedNum = false;

    final numbers = subTypes.where(
      (final type) => switch (type.className) {
        'int' || 'double' || 'num' => true,
        _ => false,
      },
    );

    if (numbers.length >= 2) {
      yield TypeResultBase('num', nullable: true);
      optimizedNum = true;
    }

    yield* subTypes.where((final type) {
      if (!optimizedNum) {
        return true;
      }

      return switch (type.className) {
        'int' || 'double' || 'num' => false,
        _ => true,
      };
    });
  }

  @override
  bool operator ==(final Object other) =>
      other is TypeResultSomeOf &&
      other.className == className &&
      other.generics == generics &&
      other.dartType == dartType;

  @override
  int get hashCode => className.hashCode + generics.hashCode + dartType.hashCode;
}

class TypeResultAnyOf extends TypeResultSomeOf {
  TypeResultAnyOf(
    super.className, {
    required super.subTypes,
    super.nullable,
  });

  @override
  String deserialize(final String object, [final String? serializerName]) =>
      '(${super.deserialize(object, serializerName)}..validateAnyOf())';

  @override
  bool operator ==(final Object other) =>
      other is TypeResultAnyOf &&
      other.className == className &&
      other.generics == generics &&
      other.dartType == dartType;

  @override
  int get hashCode => className.hashCode + generics.hashCode + dartType.hashCode;
}

class TypeResultOneOf extends TypeResultSomeOf {
  TypeResultOneOf(
    super.className, {
    required super.subTypes,
    super.nullable,
  });

  @override
  String deserialize(final String object, [final String? serializerName]) =>
      '(${super.deserialize(object, serializerName)}..validateOneOf())';

  @override
  bool operator ==(final Object other) =>
      other is TypeResultOneOf &&
      other.className == className &&
      other.generics == generics &&
      other.dartType == dartType;

  @override
  int get hashCode => className.hashCode + generics.hashCode + dartType.hashCode;
}
